@@ -7,7 +7,7 @@ DEFAULT_CONF = {
         "name": "grain_head",
         "main_log_file": "/dev/null",
         "log_file": "",
-        "listen": ":4242",
+        "listen": "tcp://:4242",
         "script": {},
     },
     "worker": {
@@ -21,10 +21,19 @@ DEFAULT_CONF = {
         "walltime": "12:00:00",
         "extra_args": [],
     },
+    "custom_system": {},
 }
 
 def load_conf(config=None):
-    conf = toml.load(config or ENV.get("GRAIN_CONFIG", "grain.toml"))
+    if config is False:
+        print("Config file is disabled, using default settings.")
+        return odict(DEFAULT_CONF)
+    config = config or ENV.get("GRAIN_CONFIG", "grain.toml")
+    try:
+        conf = toml.load(config)
+    except FileNotFoundError:
+        print(f"Cannot find Grain config file {config!r}, using default settings.")
+        conf = {}
     setdefault(conf, DEFAULT_CONF)
     return odict(conf)
 
